--author: igor29381
--v1.0
--date 15.06.17
--Kavkaz's UniversalFactory plug-in

MixerPlaceable = {};
MixerPlaceable_mt = Class(MixerPlaceable, Placeable);
InitObjectClass(MixerPlaceable, "MixerPlaceable");

function MixerPlaceable:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = MixerPlaceable_mt;
	end;
	local self = Placeable:new(isServer, isClient, mt);
	registerObjectClassName(self, "MixerPlaceable");
	return self;
end;

function MixerPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
	if not MixerPlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;
	self.pigFoodMixer = true;
	if x ~= 0 and z ~= 0 then
		local helpersNode = getChild(self.nodeId, "helpers");
		if helpersNode > 0 then
			self.helpers = {};
			for i=1, getNumOfChildren(helpersNode) do
				local node = getChildAt(helpersNode, i-1);
				local helper = Helper:new(node);
				g_currentMission:addUpdateable(helper);
				local text1 = getUserAttribute(node, "text1");
				if text1 then
					helper.text1 = g_i18n:getText(text1);
				end;
				table.insert(self.helpers, helper);
			end;
		end;
	end;
	return true;
end;

function MixerPlaceable:finalizePlacement()
	MixerPlaceable:superClass().finalizePlacement(self);
	if not self.isLoadedFromSavegame then
		local x, _, z = getWorldTranslation(self.nodeId);
		if x ~= 0 and z ~= 0 and Kavkaz and Kavkaz.version and Kavkaz.version == "2.0.6" then
			self.factory = Kavkaz.UniversalFactory:new(g_server ~= nil, g_client ~= nil);
			local i = 1;
			while true do
				local name = string.format("pigFoodMixer%d", i);
				if not Kavkaz.UniversalFactory[name] then
					break;
				end;
				i = i + 1;
			end;
			self.factory.factoryName = string.format("pigFoodMixer%d", i);
			self.factory.locationName = string.format(g_i18n:getText("pigFoodMixer"), i);
			MixerPlaceable.loadMixerAsFactory(self);
		end;
	end;
end;

function MixerPlaceable:delete()
	if self.factory then
		self.factory:unregister(true);
		self.factory:delete();
	end;
	if self.helpers then
		for i=1, #self.helpers do
			g_currentMission:removeUpdateable(self.helpers[i]);
		end;
	end;
	if g_currentMission.manuscriptPlaceables then
		g_currentMission.manuscriptPlaceables[self] = nil;
	end;
	unregisterObjectClassName(self);
	MixerPlaceable:superClass().delete(self);
end;

function MixerPlaceable.loadMixerAsFactory(mixer)
	g_currentMission.manuscriptPlaceables[mixer] = mixer;
	mixer.factory:load(mixer.nodeId, true);
	mixer.factory:register(true);
	local farmStorage = Kavkaz.UniversalFactory.Farming;
	local x1,_,z1 = getWorldTranslation(farmStorage.nodeId);
	local x,_,z = getWorldTranslation(mixer.nodeId);
	if Utils.vector2Length(x-x1, z-z1) < 50 then
		mixer.factory.farmStorageInRange = true;
	end;
end;

function MixerPlaceable:deleteFinal()
	MixerPlaceable:superClass().deleteFinal(self);
end;

function MixerPlaceable:readStream(streamId, connection)
	MixerPlaceable:superClass().readStream(self, streamId, connection);
	if self.factory then
		local factoryId = readNetworkNodeObjectId(streamId);
		self.factory:readStream(streamId, connection);
		g_client:finishRegisterObject(self.factory, factoryId);
		for i=1, #self.factory.TipTriggers do
			local trigger = self.factory.TipTriggers[i];
			local tipTriggerId = readNetworkNodeObjectId(streamId);
			trigger:readStream(streamId, connection);
			g_client:finishRegisterObject(trigger, tipTriggerId);
		end;
		for i=1, #self.factory.siloTriggers do
			local trigger = self.factory.siloTriggers[i];
			local siloTriggerId = readNetworkNodeObjectId(streamId);
			trigger:readStream(streamId, connection);
			g_client:finishRegisterObject(trigger, siloTriggerId);
		end;
	end;
end;

function MixerPlaceable:writeStream(streamId, connection)
	MixerPlaceable:superClass().writeStream(self, streamId, connection);
	if self.factory then
		writeNetworkNodeObjectId(streamId, self.factory.id);
		self.factory:writeStream(streamId, connection);
		g_server:registerObjectInStream(connection, self.factory);
		for i=1, #self.factory.TipTriggers do
			local trigger = self.factory.TipTriggers[i];
			writeNetworkNodeObjectId(streamId, trigger.id);
			trigger:writeStream(streamId, connection);
			g_server:registerObjectInStream(connection, trigger);
		end;
		for i=1, #self.factory.siloTriggers do
			local trigger = self.factory.siloTriggers[i];
			writeNetworkNodeObjectId(streamId, trigger.id);
			trigger:writeStream(streamId, connection);
			g_server:registerObjectInStream(connection, trigger);
		end;
	end;
end;

function MixerPlaceable:update(dt)
end;

function MixerPlaceable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not MixerPlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;
	local name = getXMLString(xmlFile, key.."#name");
	if g_server and name and Kavkaz and Kavkaz.version and Kavkaz.version == "2.0.6" then
		self.factory = Kavkaz.UniversalFactory:new(g_server ~= nil, g_client ~= nil);
		self.factory.factoryName = name;
		self.factory.locationName = string.format(g_i18n:getText("pigFoodMixer"), tonumber(string.sub(name, -1)));
		MixerPlaceable.loadMixerAsFactory(self);
		local xmlPath = "UniFactorySaves.xml";
		if g_currentMission.missionInfo.savegameDirectory then
			xmlPath = g_currentMission.missionInfo.savegameDirectory .. "/UniFactorySaves.xml";
		end;
		if fileExists(xmlPath) then
			local xmlFile = loadXMLFile("UniFactorySaves", xmlPath);
			local i = 0;
			while true do
				local factoryIndex = string.format("factories.factory(%d)", i);
				if not hasXMLProperty(xmlFile, factoryIndex) then break; end;
				local name = getXMLString(xmlFile, factoryIndex .. "#name");
				if name == self.factory.factoryName then
					self.factory:loadSaveData(xmlFile, factoryIndex);
					break;
				end;
				i = i + 1;
			end;
			delete(xmlFile);
		end;
	end;
	return true;
end;

function MixerPlaceable:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = MixerPlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	if self.factory then
		attributes = attributes..' name="'..self.factory.factoryName..'"';
	end;
	return attributes, nodes;
end;

registerPlaceableType("MixerPlaceable", MixerPlaceable);
