--author: igor29381
--v1.0
--date 28.06.17

StoragePlaceable = {};
StoragePlaceable_mt = Class(StoragePlaceable, Placeable);
InitObjectClass(StoragePlaceable, "StoragePlaceable");

function StoragePlaceable:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = StoragePlaceable_mt;
	end;
	local self = Placeable:new(isServer, isClient, mt);
	self.storageDirtyFlag = self:getNextDirtyFlag();
	registerObjectClassName(self, "StoragePlaceable");
	return self;
end;

function StoragePlaceable:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
	if not StoragePlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;
	local xmlFile = loadXMLFile("TempXML", xmlFilename);
	local fillTypes = getXMLString(xmlFile, "placeable.fillUnit#fillTypes");
	self.acceptedFillTypes = {};
	self.fillTypePrices = {};
	self.fillTypePriceInfo = {};
	if fillTypes then
		fillTypes = Utils.splitString(" ", fillTypes);
		for _,name in pairs(fillTypes) do
			local fillType = FillUtil.fillTypeNameToInt[name];
			if fillType then
				self.acceptedFillTypes[fillType] = true;
				self.fillTypePrices[fillType] = 0;
				self.fillTypePriceInfo[fillType] = 0;
			end;
		end;
	end;
	local capacity = Utils.getNoNil(getXMLInt(xmlFile, "placeable.fillUnit#capacity"), 100000);
	if x ~= 0 and z ~= 0 then
		self.storageUnits = {};
		local storageUnitsNode = getChild(self.nodeId, "storageUnits");
		if storageUnitsNode > 0 then
			for i=1, getNumOfChildren(storageUnitsNode) do
				local unitId = getChildAt(storageUnitsNode, i-1);
				local unit = StorageUnit:new(unitId, self);
				g_currentMission:addNonUpdateable(unit);
				unit.fillUnit.capacity = capacity;
				unit.index = i;
				table.insert(self.storageUnits, unit);
			end;
		end;
		local appearsOnPDA = Utils.getNoNil(getXMLBool(xmlFile, "placeable.appearsOnPDA"), false);
		if appearsOnPDA then
			local visitPoint = getChild(self.nodeId, "visitPoint");
			if visitPoint > 0 then
				x, _, z = getWorldTranslation(visitPoint);
			end;
			local width, height = getNormalizedScreenValues(10, 10);
			local uvs = getNormalizedUVs({531, 940, 46, 46});
			self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("", "", g_baseUIFilename, uvs, nil, x, z, width, height, false, false, true, nil, true);
		end;
		if g_currentMission.manuscriptPlaceables then
			g_currentMission.manuscriptPlaceables[self] = self;
		end;
	end;
	delete(xmlFile);
	self.enableUpdateClients = 0;
	self.storagePlaceable = true;
	return true;
end;

function StoragePlaceable:finalizePlacement()
	StoragePlaceable:superClass().finalizePlacement(self);
	if self.storageUnits then
		for i=1, #self.storageUnits do
			local unit = self.storageUnits[i];
			unit:finalizePlacement();
			if not self.isLoadedFromSavegame then
				unit:setCurrentFillType(FillUtil.FILLTYPE_UNKNOWN);
			end;
		end;
	end;
end;

function StoragePlaceable:delete()
	if self.mapHotspot then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot);
	end;
	if g_currentMission.manuscriptPlaceables then
		g_currentMission.manuscriptPlaceables[self] = nil;
	end;
	if self.storageUnits then
		for i=1, #self.storageUnits do
			local unit = self.storageUnits[i];
			if unit.helper then
				g_currentMission:removeUpdateable(unit.helper);
				unit.helper = nil;
			end;
			g_currentMission:removeNonUpdateable(unit);
			unit:delete();
		end;
	end;
	unregisterObjectClassName(self);
	StoragePlaceable:superClass().delete(self);
end;

function StoragePlaceable:deleteFinal()
	StoragePlaceable:superClass().deleteFinal(self);
end;

function StoragePlaceable:readStream(streamId, connection)
	StoragePlaceable:superClass().readStream(self, streamId, connection);
	if connection:getIsServer() then
		for i=1, #self.storageUnits do
			local unit = self.storageUnits[i];
			local tipTriggerId = readNetworkNodeObjectId(streamId);
			unit.tipTrigger:readStream(streamId, connection);
			g_client:finishRegisterObject(unit.tipTrigger, tipTriggerId);
			local siloTriggerId = readNetworkNodeObjectId(streamId);
			unit.siloTrigger:readStream(streamId, connection);
			g_client:finishRegisterObject(unit.siloTrigger, siloTriggerId);
			local fillType = streamReadInt8(streamId);
			unit.fillUnit.level = streamReadInt32(streamId);
			unit:setCurrentFillType(fillType);
			unit:heapMoving();
		end;
	end;
end;

function StoragePlaceable:writeStream(streamId, connection)
	StoragePlaceable:superClass().writeStream(self, streamId, connection);
	if not connection:getIsServer() then
		for i=1, #self.storageUnits do
			local unit = self.storageUnits[i];
			writeNetworkNodeObjectId(streamId, unit.tipTrigger.id);
			unit.tipTrigger:writeStream(streamId, connection);
			g_server:registerObjectInStream(connection, unit.tipTrigger);
			writeNetworkNodeObjectId(streamId, unit.siloTrigger.id);
			unit.siloTrigger:writeStream(streamId, connection);
			g_server:registerObjectInStream(connection, unit.siloTrigger);
			streamWriteInt8(streamId, unit.fillUnit.currentFillType);
			streamWriteInt32(streamId, math.ceil(unit.fillUnit.level));
		end;
	end;
end;

function StoragePlaceable:readUpdateStream(streamId, timestamp, connection)
	StoragePlaceable:superClass().readUpdateStream(self, streamId, timestamp, connection);
	if connection:getIsServer() then
		local index = streamReadInt8(streamId);
		local unit = self.storageUnits[index];
		local level = unit.fillUnit.level;
		local fillType = streamReadInt8(streamId);
		unit.fillUnit.level = streamReadInt32(streamId);
		if level == 0 or unit.fillUnit.level == 0 then
			unit:setCurrentFillType(fillType);
		end;
		unit:heapMoving();
	end;
end;

function StoragePlaceable:writeUpdateStream(streamId, connection, dirtyMask)
	StoragePlaceable:superClass().writeUpdateStream(self, streamId, connection, dirtyMask);
	if not connection:getIsServer() then
		local unit = self.storageUnits[self.enableUpdateClients].fillUnit;
		streamWriteInt8(streamId, self.enableUpdateClients);
		streamWriteInt8(streamId, unit.currentFillType);
		streamWriteInt32(streamId, math.ceil(unit.level));
	end;
end;

function StoragePlaceable:update(dt)
end;

function StoragePlaceable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not StoragePlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;
	for i=1, #self.storageUnits do
		local unit = self.storageUnits[i];
		local unitKey = string.format("%s.storageUnit%d", key, i);
		local fillType = getXMLString(xmlFile, unitKey.."#fillType");
		fillType = FillUtil.fillTypeNameToInt[fillType];
		local fillLevel = Utils.getNoNil(getXMLFloat(xmlFile, unitKey.."#fillLevel"), 0);
		if fillLevel > 0 then
			unit:updateFillLevel(fillLevel, fillType);
		else
			unit:setCurrentFillType(FillUtil.FILLTYPE_UNKNOWN);
		end;
		unit:heapMoving();
	end;
	return true;
end;

function StoragePlaceable:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = StoragePlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	for i=1, #self.storageUnits do
		local unit = self.storageUnits[i].fillUnit;
		if i>1 then
			nodes = nodes.."\n";
		end;
		nodes = nodes.. nodeIdent..'<storageUnit'..i..' fillType="'..FillUtil.fillTypeIntToName[unit.currentFillType]..'" fillLevel="'..unit.level..'" />';
	end;
	return attributes, nodes;
end;

registerPlaceableType("StoragePlaceable", StoragePlaceable);
